<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="Help Section";
$page_message="Read $PRODUCT_NAME documentation, see advanced usage tips for expert users, and find detailed answers to frequently asked questions.";
$page_message_class="alert alert-info";


//display help topics
function returnHelpTopicsArray($PRODUCT_NAME, $ROOT_URL, $array_to_use, $help_topic_class)
    {
    $root_array=array();

    //using application
    $using_application=array(
    "Getting started"=>"Before you can start protecting your script with $PRODUCT_NAME, it only takes 3 easy steps: adding product, adding client (optional), and adding license itself. For your convenience, all the steps are fully explained below.",

    "Adding new product"=>"The first and most obvious step to license a product is to create the product itself. Open the <i>Products > Add New Product</i> tab to create your product. Product name and SKU fields are mandatory and must be unique (different products with same name and/or SKU can't exist), while other fields are optional. The number of products is unlimited.",

    "Editing existing product"=>"All previously added products can be edited using the <i>Products > View Products</i> tab. Click desired product name to make any changes needed. Even if license for a particular product is in use, all the details (even product name or SKU) can be modified without affecting existing installations.<br><br>
    Setting product status to Inactive will deactivate all licenses issued for this product. For example, if there are 10 clients with active licenses for particular product, all 10 licenses will not be available to use anymore. This option should only be used if you want to prevent users from using a particular product for some reason.",

    "Adding new client"=>"Once product is created, add a new client. Open the <i>Clients > Add New Client</i> tab to add a client who will use licensed product. All fields are mandatory. Since email address is used to identify client and verify his license, it must be unique (different clients with same email address can't exist). The number of clients is unlimited.<br><br>
    While it's always recommended to create client's profile before issuing a license, there might be a scenario when you don't know client's details (such as name and email) or want to issue anonymous license. In case that happens, just skip this step.",

    "Editing existing client"=>"Just like products, existing clients can also be edited using the <i>Clients > View Clients</i> tab. Click desired client name to make any changes needed. Client first and last name can be modified at any time, but modifying licensed email address after license is issued will cause a failure of client's license verification. In order to continue using his license issued under old email address, client will need to update his licensed email address in installed script. Just as you would expect, setting client status to Inactive will deactivate all licenses issued for selected client. For example, if client has 5 separate licenses for 5 different products, all 5 licenses will become inactive.",

    "Adding new license"=>"Finally, it's time to issue a new license. Open the <i>Licenses > Add New License</i> tab to add a new license. When client's details are known, select client and product from the list. In case you don't know client's name and email (so you can't create client's profile), select product from the list and enter unique license code. This way, anyone having this code will be able to activate his copy of protected script.<br><br>
    Only product and client (for personal licenses) or license code (for anonymous licenses) fields are mandatory, while optional fields can be used to bind license to specific IP address and/or domain, limit the number of allowed installations, or even cause license to expire at specific date. More on that below.<br><br>
    If IP address is entered, application will only work on a specific IP address. Once IP is changed, license check will fail. If domain is entered, application will only work on a specific domain. Domain should always be entered as clientdomain.com or clientdomain.com/path (it should never include / symbol at the end). If licensed domain is entered as clientdomain.com, product will work on clientdomain.com and clientdomain.com/any/directory. If licensed domain is entered as clientdomain.com/path, product will only work on clientdomain.com/path. Sure enough, both IP and domain fields can be filled at the same time, so application will only work if IP address matches domain (for example, if IP address 1.1.1.1 and domain clientdomain.com is set, but clientdomain.com actually points to 1.1.1.2, application will not work). It's also possible to add multiple licensed IPs and/or domains by separating them with comma (,) symbol.<br><br>
    An option to always require a domain can also be used; it will block all installation attempts on invalid domains (in other words, installations on http://localhost or http://1.1.1.1 will fail).<br><br>
    If installations limit is set, client will not be able to perform more installations once limit is reached. For example, if current limit is 2 installations, 3rd installation will only work when one of previous installations is deleted using the <i>View Installations</i> tab (the limit is set per license). If installations limit is set, IP address and/or domain fields can be left empty, since client will not be able to perform any more installations anyway (no matter domain and/or IP).<br><br>
    If expiration date is set, application will stop working after this date (expiration date can be updated at any time). Additionally, it's possible to set updates and support expiration dates. If updates and/or support expiration date is set, application will continue working as long as the license is active, but updates and/or support will not be available after this date.",

    "Editing existing license"=>"All issued licenses be edited using the <i>Licenses > View Licenses</i> tab. Click desired license to make any changes needed. If license is in use, be aware that modifying license data might cause license verification to fail. For example, if software is installed on clientdomain.com and licensed domain is changed to clientdomain.com/path, existing installation will stop working. In other hand, license modification is a must when some details (such as client's IP address) was changed, so client can continue using protected script.<br><br>
    For very obvious security reasons, licensed product or license owner can not be changed after license is issued. If existing client needs a license for another product, just terminate his existing license (optional) and issue a new one. Setting license status to Inactive will deactivate this license. For example, if client has 3 separate licenses for 3 different products, deactivating one license will prevent client from using a particular product, but allow him using other two products he still has active licenses for.",

    "Editing existing installation"=>"All installations be edited using the <i>Installations > View Installations</i> tab. Click desired installation to make any changes needed. For security reasons, only installation IP address and status can be modified. An existing installation should only be modified if client's IP address was changed and license verification failed. Otherwise, installation details should never be modified. Deactivating or completely deleting an existing installation will cause license verification to fail and client will need to re-install protected application.",

    "Customizing server notifications and emails"=>"$PRODUCT_NAME allows displaying custom notifications in your application for each event (failed license check, expired license, etc.) and/or sending automated reminder emails about expiring license, updates, and support. Open the <i>Server Notifications > Customize Notifications</i> tab to modify these notifications or the <i>Server Notifications > Customize Emails</i> tab to modify reminder emails. The full list of supported variables is below (all variables are sorted alphabetically):<br><br>
    <table class=\"custom_table_minimal table table-bordered table-striped\">
            <thead>
              <tr>
                 <th>Variable Name</th>
                 <th>Variable Value</th>
                 <th>Notes</th>
              </tr>
            </thead>
            <tbody>
              <tr>
                 <td>%CLIENT_EMAIL%</td>
                 <td>Client's email address</td>
                 <td>Only available for personal licenses</td>
              </tr>
              <tr>
                 <td>%CLIENT_FNAME%</td>
                 <td>Client's first name</td>
                 <td>Only available for personal licenses</td>
              </tr>
              <tr>
                 <td>%CLIENT_LNAME%</td>
                 <td>Client's last name</td>
                 <td>Only available for personal licenses</td>
              </tr>
              <tr>
                 <td>%IP_ADDRESS%</td>
                 <td>IP address of user's machine</td>
                 <td>Not available in reminder emails</td>
              </tr>
              <tr>
                 <td>%LICENSE_CANCEL_DATE%</td>
                 <td>License cancellation date</td>
                 <td>Only available for cancelled licenses</td>
              </tr>
              <tr>
                 <td>%LICENSE_CODE%</td>
                 <td>License code</td>
                 <td>Only available for anonymous licenses</td>
              </tr>
              <tr>
                 <td>%LICENSE_EXPIRE_DATE%</td>
                 <td>License expiration date</td>
                 <td>Only available for time-expiring licenses</td>
              </tr>
              <tr>
                 <td>%LICENSE_LIMIT%</td>
                 <td>Number of allowed installations</td>
                 <td>Only available for licenses with installations limit</td>
              </tr>
              <tr>
                 <td>%LICENSE_SUPPORT_DATE%</td>
                 <td>Support expiration date</td>
                 <td>Only available for support-expiring licenses</td>
              </tr>
              <tr>
                 <td>%LICENSE_UPDATES_DATE%</td>
                 <td>Updates expiration date</td>
                 <td>Only available for updates-expiring licenses</td>
              </tr>
              <tr>
                 <td>%PRODUCT_DESCRIPTION% </td>
                 <td>Product description</td>
                 <td>Not available for 'Invalid License' notification</td>
              </tr>
              <tr>
                 <td>%PRODUCT_ID% </td>
                 <td>Internal product ID</td>
                 <td>Not recommended to display for security reasons. Not available for 'Invalid License' notification</td>
              </tr>
              <tr>
                 <td>%PRODUCT_TITLE% </td>
                 <td>Product title</td>
                 <td>Not available for 'Invalid License' notification</td>
              </tr>
              <tr>
                 <td>%PRODUCT_URL_DOWNLOAD% </td>
                 <td>Product download URL</td>
                 <td>Not available for 'Invalid License' notification</td>
              </tr>
              <tr>
                 <td>%PRODUCT_URL_HOMEPAGE% </td>
                 <td>Product homepage URL</td>
                 <td>Not available for 'Invalid License' notification</td>
              </tr>
              <tr>
                 <td>%PRODUCT_VERSION% </td>
                 <td>Current product version</td>
                 <td>Not available for 'Invalid License' notification</td>
              </tr>
              <tr>
                 <td>%ROOT_URL%</td>
                 <td>URL where application is (being) installed</td>
                 <td>Not available in reminder emails</td>
              </tr>
            </tbody>
          </table>",
    );


    //using API
    $using_api=array(
    "Enabling and configuring API"=>"With the help of $PRODUCT_NAME API, you can perform any action from a remote script. Just enable API using the <i>Software Settings > Advanced Settings</i> tab and send all requests to <i>$ROOT_URL/apl_api/api.php</i>.<br><br>
    Before API can be accessed, a new API key should be created using the <i>API Keys > Add New API Key</i> tab. The software allows creating multiple API keys with different permissions; therefore, you can grant different permissions for different keys. Optionally, you can accept API requests only from specific IP address(es), so connections from other IPs will be blocked. Any API key can be activated/deactivated at any time.<br><br>
    Additionally, you can secure the /apl_api directory using <a href='http://www.htaccesstools.com/articles/password-protection/' target='_blank'>htaccess password protection</a> (you will need to add authorization headers to all API requests in this case). If you don't plan using API, it should be disabled.",

    "Formatting and sending API requests"=>"All API requests should be sent to /apl_api/api.php using <code>POST</code> method and include unique API secret along with additional data you want to submit. Here's a basic example on how API request should look like:<br><br>
    API URL:<br>
    <pre>$ROOT_URL/apl_api/api.php</pre><br>
    Post Data:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=FUNCTION_TO_CALL&var1=value1&var2=value2...</pre><br>
    API will always return a success/failure message for each request. In case some action fails, the cause of error will be included as well. If no message is returned at all, it means your request was rejected. It might happen when data is submitted using GET instead of POST method, API URL is invalid, or some parameters are missing.<br><br>
    Some API functions require a status and/or date to be included, which is easy to do: 1 means active (enabled), 0 means inactive (disabled). For example, if you add a product and set its status to 0, product will be stored in database, but it won't be available for licensing.<br><br>
    Active (enabled): <code>1</code><br>
    Inactive (disabled): <code>0</code><br><br>
    Date should always be formatted using a locale neutral format (as specified by <a href=\"https://en.wikipedia.org/wiki/ISO_8601\" target=\"_blank\">ISO 8601</a>): <code>yyyy-mm-dd</code>.<br><br>
    In case any record needs to be deleted, an extra parameter <code>delete_record</code> with value <code>1</code> should be added:<br><br>
    Delete Record: <code>delete_record=1</code>",

    "API parameters List"=>"All the API parameters you will ever use are self-explanatory. However, if you find some variable not clear, refer to this list for more details (all variables are sorted alphabetically):<br><br>
    <table class=\"custom_table_minimal table table-bordered table-striped\">
            <thead>
              <tr>
                 <th>Parameter Name</th>
                 <th>Description</th>
                 <th>Notes</th>
              </tr>
            </thead>
            <tbody>
              <tr>
                 <td>banned_host_comments</td>
                 <td>Optional blocked IP comments</td>
                 <td></td>
              </tr>
              <tr>
                 <td>banned_host_id</td>
                 <td>Numeric blocked IP ID</td>
                 <td></td>
              </tr>
              <tr>
                 <td>banned_host_ip</td>
                 <td>IP address to block</td>
                 <td></td>
              </tr>
              <tr>
                 <td>client_email</td>
                 <td>Client email address</td>
                 <td>Must be unique</td>
              </tr>
              <tr>
                 <td>client_fname</td>
                 <td>Client first name</td>
                 <td></td>
              </tr>
              <tr>
                 <td>client_id</td>
                 <td>Numeric client ID</td>
                 <td>Can be parsed from search API</td>
              </tr>
              <tr>
                 <td>client_lname</td>
                 <td>Client last name</td>
                 <td></td>
              </tr>
              <tr>
                 <td>client_status</td>
                 <td>Client status</td>
                 <td>Active: <code>1</code>, inactive: <code>0</code></td>
              </tr>
              </tr>
              <tr>
                 <td>delete_record</td>
                 <td>Delete particular record</td>
                 <td>Delete: <code>1</code></td>
              </tr>
              <tr>
                 <td>installation_id</td>
                 <td>Numeric installation ID</td>
                 <td>Can be parsed from search API</td>
              </tr>
              <tr>
                 <td>installation_ip</td>
                 <td>Installation IP address</td>
                 <td></td>
              </tr>
              <tr>
                 <td>installation_status</td>
                 <td>Installation IP address</td>
                 <td>Active: <code>1</code>, inactive: <code>0</code></td>
              </tr>
              <tr>
                 <td>license_code</td>
                 <td>License code</td>
                 <td>Only required for anonymous licenses</td>
              </tr>
              <tr>
                 <td>license_comments</td>
                 <td>Optional license comments</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_domain</td>
                 <td>Domain(s) to bind license to</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_expire_date</td>
                 <td>Date after which license will expire</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_id</td>
                 <td>Numeric license ID</td>
                 <td>Can be parsed from search API</td>
              </tr>
              <tr>
                 <td>license_ip</td>
                 <td>IP address(es) to bind license to</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_limit</td>
                 <td>Maximum number of installations</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_order_number</td>
                 <td>Order number of license purchase</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_require_domain</td>
                 <td>Option to always require a domain</td>
                 <td>Yes: <code>1</code>, no: <code>0</code></td>
              </tr>
              <tr>
                 <td>license_status</td>
                 <td>License status</td>
                 <td>Active: <code>1</code>, inactive: <code>0</code></td>
              </tr>
              <tr>
                 <td>license_support_date</td>
                 <td>Date after which support will expire</td>
                 <td></td>
              </tr>
              <tr>
                 <td>license_updates_date</td>
                 <td>Date after which updates will not be available</td>
                 <td></td>
              </tr>
              <tr>
                 <td>product_description</td>
                 <td>Short product description</td>
                 <td></td>
              </tr>
              <tr>
                 <td>product_envato_id</td>
                 <td>Product Envato ID</td>
                 <td>Only used for products sold on Envato</td>
              </tr>
              <tr>
                 <td>product_id</td>
                 <td>Numeric product ID</td>
                 <td>Can be parsed from search API</td>
              </tr>
              <tr>
                 <td>product_sku</td>
                 <td>Product stock keeping unit</td>
                 <td>Must be unique</td>
              </tr>
              <tr>
                 <td>product_status</td>
                 <td>Product status</td>
                 <td>Active: <code>1</code>, inactive: <code>0</code></td>
              </tr>
              <tr>
                 <td>product_title</td>
                 <td>Product name</td>
                 <td>Must be unique</td>
              </tr>
              <tr>
                 <td>product_url_download</td>
                 <td>Product download URL</td>
                 <td></td>
              </tr>
              <tr>
                 <td>product_url_homepage</td>
                 <td>Product homepage URL</td>
                 <td></td>
              </tr>
              <tr>
                 <td>product_version</td>
                 <td>Product version</td>
                 <td></td>
              </tr>
              <tr>
                 <td>search_keyword</td>
                 <td>Keyword(s) to be used for search</td>
                 <td></td>
              </tr>
              <tr>
                 <td>search_type</td>
                 <td>API search type</td>
                 <td>Available values: <code>product</code>, <code>client</code>, <code>license</code>, <code>installation</code></td>
              </tr>
            </tbody>
          </table>",

    "Adding new product"=>"API Function: <code>products_add</code><br>
    Required Parameters: <code>product_title</code>, <code>product_sku</code>, <code>product_status</code><br>
    Optional Parameters: <code>product_description</code>, <code>product_url_homepage</code>, <code>product_url_download</code>, <code>product_version</code>, <code>product_envato_id</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=products_add&product_title=My_Product_Name&product_sku=MPN&product_status=1&product_description=some_description&product_url_homepage=$ROOT_URL&product_url_download=$ROOT_URL/file.zip&product_version=1.0&product_envato_id=123456</pre>",

    "Editing existing product"=>"API Function: <code>products_edit</code><br>
    Required Parameters: <code>product_id</code>, <code>product_title</code>, <code>product_sku</code>, <code>product_status</code><br>
    Optional Parameters: <code>product_description</code>, <code>product_url_homepage</code>, <code>product_url_download</code>, <code>product_version</code>, <code>product_envato_id</code>, <code>delete_record</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=products_edit&product_id=1&product_title=My_Product_Name&product_sku=MPN&product_status=1&product_description=some_description&product_url_homepage=$ROOT_URL&product_url_download=$ROOT_URL/file.zip&product_version=1.0&product_envato_id=123456</pre>",

    "Adding new client"=>"API Function: <code>clients_add</code><br>
    Required Parameters: <code>client_fname</code>, <code>client_lname</code>, <code>client_email</code>, <code>client_status</code><br>
    Optional Parameters: <code>N/A</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=clients_add&client_fname=First_Name&client_lname=Last_Name&client_email=client@email.com&client_status=1</pre>",

    "Editing existing client"=>"API Function: <code>clients_edit</code><br>
    Required Parameters: <code>client_id</code>, <code>client_fname</code>, <code>client_lname</code>, <code>client_email</code>, <code>client_status</code><br>
    Optional Parameters: <code>delete_record</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=clients_edit&client_id=1&client_fname=First_Name&client_lname=Last_Name&client_email=client@email.com&client_status=1</pre>",

    "Adding new license"=>"This one is a bit more tricky, since it requires submitting both product and client IDs stored in the database. How do you get required IDs? Using search API! Since detailed instructions on how to use search API is available in the Help Section itself, let's assume you already know how to find records.<br><br>
    API Function: <code>licenses_add</code><br>
    Required Parameters (personal license): <code>product_id</code>, <code>client_id</code>, <code>license_require_domain</code>, <code>license_status</code><br>
    Optional Parameters (personal license): <code>license_order_number</code>, <code>license_ip</code>, <code>license_domain</code>, <code>license_limit</code>, <code>license_expire_date</code>, <code>license_updates_date</code>, <code>license_support_date</code>, <code>license_comments</code><br><br>
    Post Data Example (personal license):<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=licenses_add&product_id=1&client_id=1&license_require_domain=1&license_status=1&license_order_number=order123&license_ip=192.168.1.1&license_domain=clientdomain.com&license_limit=3&license_expire_date=".date("Y-m-d", strtotime("+1 year"))."&license_updates_date=".date("Y-m-d", strtotime("+9 months"))."&license_support_date=".date("Y-m-d", strtotime("+6 months"))."&license_comments=something</pre><br><br>
    Required Parameters (anonymous license): <code>product_id</code>, <code>license_code</code>, <code>license_require_domain</code>, <code>license_status</code><br>
    Optional Parameters (anonymous license): <code>license_order_number</code>, <code>license_ip</code>, <code>license_domain</code>, <code>license_limit</code>, <code>license_expire_date</code>, <code>license_updates_date</code>, <code>license_support_date</code>, <code>license_comments</code><br><br>
    Post Data Example (anonymous license):<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=licenses_add&product_id=1&license_code=code123456&license_require_domain=1&license_status=1&license_order_number=order123&license_ip=192.168.1.1&license_domain=clientdomain.com&license_limit=3&license_expire_date=".date("Y-m-d", strtotime("+1 year"))."&license_updates_date=".date("Y-m-d", strtotime("+9 months"))."&license_support_date=".date("Y-m-d", strtotime("+6 months"))."&license_comments=something</pre>",

    "Editing existing license"=>"API Function: <code>licenses_edit</code><br>
    Required Parameters (personal license): <code>product_id</code>, <code>client_id</code>, <code>license_id</code>, <code>license_require_domain</code>, <code>license_status</code><br>
    Optional Parameters (personal license): <code>license_order_number</code>, <code>license_ip</code>, <code>license_domain</code>, <code>license_limit</code>, <code>license_expire_date</code>, <code>license_updates_date</code>, <code>license_support_date</code>, <code>license_comments</code>, <code>delete_record</code><br><br>
    Post Data Example (personal license):<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=licenses_edit&product_id=1&client_id=1&license_id=1&license_require_domain=1&license_status=1&license_order_number=order123&license_ip=192.168.1.1&license_domain=clientdomain.com&license_limit=3&license_expire_date=".date("Y-m-d", strtotime("+1 year"))."&license_updates_date=".date("Y-m-d", strtotime("+9 months"))."&license_support_date=".date("Y-m-d", strtotime("+6 months"))."&license_comments=something</pre><br><br>
    Required Parameters (anonymous license): <code>product_id</code>, <code>license_id</code>, <code>license_code</code>, <code>license_require_domain</code>, <code>license_status</code><br>
    Optional Parameters (anonymous license): <code>license_order_number</code>, <code>license_ip</code>, <code>license_domain</code>, <code>license_limit</code>, <code>license_expire_date</code>, <code>license_updates_date</code>, <code>license_support_date</code>, <code>license_comments</code>, <code>delete_record</code><br><br>
    Post Data Example (anonymous license):<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=licenses_edit&product_id=1&license_id=1&license_code=code123456&license_require_domain=1&license_status=1&license_order_number=order123&license_ip=192.168.1.1&license_domain=clientdomain.com&license_limit=3&license_expire_date=".date("Y-m-d", strtotime("+1 year"))."&license_updates_date=".date("Y-m-d", strtotime("+9 months"))."&license_support_date=".date("Y-m-d", strtotime("+6 months"))."&license_comments=something</pre>",

    "Editing existing installation"=>"API Function: <code>installations_edit</code><br>
    Required Parameters: <code>installation_id</code>, <code>installation_ip</code>, <code>installation_status</code><br>
    Optional Parameters: <code>delete_record</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=installations_edit&installation_id=1&installation_ip=192.168.1.1&installation_status=1</pre>",

    "Adding new banned host"=>"API Function: <code>banned_hosts_add</code><br>
    Required Parameters: <code>banned_host_ip</code><br>
    Optional Parameters: <code>banned_host_comments</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=banned_hosts_add&banned_host_ip=192.168.1.1&banned_host_comments=Unauthorized</pre>",

    "Editing existing banned host"=>"API Function: <code>banned_hosts_edit</code><br>
    Required Parameters: <code>banned_host_id</code>, <code>banned_host_ip</code><br>
    Optional Parameters: <code>banned_host_comments</code>, <code>delete_record</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=banned_hosts_edit&banned_host_id=1&banned_host_ip=192.168.1.1&banned_host_comments=Unauthorized</pre>",

    "Searching for data"=>"API Function: <code>search</code><br>
    Required Parameters: <code>search_type</code>, <code>search_keyword</code><br>
    Optional Parameters: <code>N/A</code><br><br>
    Post Data Example:<br>
    <pre>api_key_secret=UNIQUE_API_SECRET&api_function=search&search_type=product&search_keyword=My_Product_Name</pre><br>
    Parameter <code>search_type</code> can contain one of these values: <code>product</code>, <code>client</code>, <code>license</code>, <code>installation</code>, <code>callback</code>. Parameter <code>search_keyword</code> can contain the exact search term or a part of it.<br><br>
    You can find matching records by one of these values:<br><br>
    Product: <code>product name</code>, <code>SKU</code><br>
    Client: <code>first name</code>, <code>last name</code>, <code>email address</code><br>
    License: <code>email address</code>, <code>license code</code><br>
    Installation: <code>email address</code>, <code>license code</code>, <code>installation domain</code>, <code>installation IP</code><br>
    Callback: <code>email address</code>, <code>license code</code>, <code>callback domain</code>, <code>callback IP</code><br><br>
    Search results are always returned using JSON format.",

    "Ready to use code for connecting to API"=>"If you don't know how to make POST requests to remote server, you can use this code to connect to API:<br><br>
    <pre>function simplePost(\$post_url, \$post_info)
    {
    \$ch=curl_init();
    curl_setopt(\$ch, CURLOPT_URL, \$post_url);
    curl_setopt(\$ch, CURLOPT_POST, 1);
    curl_setopt(\$ch, CURLOPT_POSTFIELDS, \$post_info);
    curl_setopt(\$ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt(\$ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt(\$ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt(\$ch, CURLOPT_FOLLOWLOCATION, 1);
    \$result=curl_exec(\$ch);
    curl_close(\$ch);

    return \$result;
    }</pre><br>
    Should you want to add a new product, use this code:<br><br>
    <pre>simplePost(\"$ROOT_URL/apl_api/api.php\", \"api_key_secret=UNIQUE_API_SECRET&api_function=products_add&product_title=My_Product_Name&product_sku=MPN&product_status=1\")</pre>",
    );



    //protecting your script
    $protecting_your_script=array(
    "General requirements for licensing engine"=>"$PRODUCT_NAME works on any server that supports <a href=\"https://secure.php.net/\" target=\"_blank\">PHP</a> 5.5 or higher along with its most popular extensions: <a href=\"https://secure.php.net/manual/en/book.mysqli.php\" target=\"_blank\">MySQLi</a>, <a href=\"https://secure.php.net/manual/en/book.curl.php\" target=\"_blank\">cURL</a> and <a href=\"https://secure.php.net/manual/en/book.openssl.php\" target=\"_blank\">OpenSSL</a>. Since 99% of hosting providers (even free ones) have these extensions enabled by default, $PRODUCT_NAME is virtually compatible with any server around.<br><br>
    End user's machine (the one running script protected by $PRODUCT_NAME) has no extra requirements; it only needs to support the same PHP extensions as $PRODUCT_NAME server. If user server doesn't support MySQLi, the application can still work as long as cURL and OpenSSL are installed.",

    "Integrating $PRODUCT_NAME into your script"=>"This type of information is only available outside administration dashboard for security reasons. A detailed integration guide is included in the $PRODUCT_NAME installation package; see '<i>/DOCUMENTATION</i>' directory for step-by-step instructions and full documentation.<br><br>
    The installation package also contains 2 fully working demo programs protected by $PRODUCT_NAME; feel free to explore their source code for even better understanding how system works.",

    "Automatically updating your script"=>"It's possible to automatically install and update protected script on user's machine with the help of <a href=\"https://codecanyon.net/item/php-auto-update-script/21747312?ref=phpmillion\" target=\"_blank\">PHP Auto Update Script</a>.<br><br>
    PHP Auto Update Script downloads and installs a desired version of protected application on user's machine and keeps it up to date, so you can use it to add version check features into your application, create online installers, upgrade and downgrade current installations, etc. The module is fully compatible with $PRODUCT_NAME; therefore, you can license and automatically update yor products at the same time.",
    );



    //faq
    $faq=array(
    "What is additional authentication protection in administrator profile?"=>"This option hardens the security of administrator account by constantly checking IP address and browser details. Technically speaking, if someone manages to steal authentication cookie from your computer and loads it into his browser to access your profile, the cookie will be terminated and login will fail. However, be aware that login will also fail if your own IP address or any bit of browser details (like version number) gets changed. But don't worry - all you need to do is login again, so new authentication credentials are created for you.",

    "What's the difference between personal and anonymous license?"=>"Personal licenses are issued when basic client's details (such as name and email) are known, so client's profile can be created and stored in the database. In order to activate product, client will only need to enter his licensed email address during installation; therefore, this method is always recommended for easier licenses management.<br><br>
    However, there might be some scenarios when you want to issue a license to anyone who knows activation code (for example, when you sell license keys without asking for personal buyer's information). This is called an anonymous license and doesn't require a client's profile to be created. Instead of this, you simply generate a unique license code when issuing a new license. Once someone enters this code during installation, product gets activated. Needless to say, the code must match licensed product - if code is generated for product A, it will not work with any other product.<br><br>
    No matter which method you choose, all the additional licensing options work equally.",

    "Can both email address and license code based license be issued?"=>"License can only be either personal (client-based) or anonymous (code-based), which means issuing both email address and code based license is not possible.<br><br>However, it's perfectly fine to ask client to enter his email address during installation when anonymous license is issued. It's the default way $PRODUCT_NAME itself works: email address is used to login to the administration dashboard, and code is used to verify license (don't worry, $PRODUCT_NAME will determine if license is personal or anonymous).",

    "What are limitations for license code?"=>"None. You are welcome to use any format for license code to suit your personal needs. You can even use order number as license code (so client can activate his script by entering his order ID). The only requirement for code is to be unique for each license. For your convenience, $PRODUCT_NAME will return duplicated data error if some code already exists in the database.",

    "Can multiple IPs and/or domains be assigned to the same license?"=>"Yes, a single license can have an unlimited number of allowed IPs and/or domains. Just separate each authorized IP/domain with comma (,) symbol and you are done.",

    "How installations limit works?"=>"Limiting the number of total installations is the easiest way to add maximum protection to your script without a need to bind license to IP address and/or domain. For example, if you don't want end user to share his copy of script with 3rd parties, but you don't know what IP and/or domain user will install script on, simply set installations limit to 1. This way, user will be able to install script on any IP/domain, but if he tries to install another copy, new installation will be blocked.<br><br>
    If user decides he wants to have script running on another IP/domain later, old installation needs to be deleted by you using the <i>Installations > View Installations</i> tab, so the license becomes \"available\" and another installation can be performed again. If user wants to re-install script on the same IP/domain, he can do so at any time; if the same old system is used, $PRODUCT_NAME allows an unlimited number of re-installations on the same computer. However, if the same computer is used, but IP address and/or domain is changed, re-installation will be counted as a new installation just as you would expect from a sophisticated protection algorithm.<br><br>
    The limit can also be modified after some installations were performed. For example, if user performed 3 installations, and limit is set to 1 later, all installations will stop working until 2 installations are removed using the <i>Installations > View Installations</i> tab.",

    "What's the difference between license, support and updates expiration?"=>"License expiration is used to set a date after which protected script will stop working and user will need to renew his license.<br><br>
    Support expiration is used to set a date after which protected script will continue working, but user will need to renew his support contract in order to receive further support from you. If you provide scripts with time-limited support service, you can use this feature to check if user, who asks for some assistance, is eligible for this. You can also automatically check user's support contract status in your support ticket system by calling $PRODUCT_NAME API.<br><br>
    Updates expiration is used to set a a date after which protected script will continue working, but no updates will be available. This way, you can ask users to pay for updates after X days/weeks/months/years since original purchase.<br><br>
    Needless to say, $PRODUCT_NAME allows setting all the dates independently from each other.",

    "What's the difference between license uninstallation and termination in protected script?"=>"License uninstallation is a graceful way for user to reset/uninstall his own license. Uninstalling license means that user will be able to re-install protected script on different domain later. Uninstallation does not delete user files or database; it simply removes installation details from your licensing server and marks license as \"free\". Even if no files are deleted, protected script stops working right after license is uninstalled. This way, you can add uninstaller to your script, so users can release their own licenses without interrupting you.<br<br>
    License termination is anti-hacking feature, which is only executed when hacking attempt is detected or user tries using protected script after his license was marked as cancelled by developer. This feature automatically deletes all the files of protected script in user's server and removes all the data from MySQL database (if script uses it).",

    "What are Smart Reports and how do they work?"=>"This option prevents excessive callbacks and reports from being stored in $PRODUCT_NAME database when same client connects to your server multiple times a day. By default, $PRODUCT_NAME logs every license verification, so you can track protected script usage. However, if some client has an inactive license, he might generate hundreds of excessive reports a day. Let's say client runs some website with your script installed, and script is not licensed anymore (or you accidentally enabled forced license check). Every time someone visits client's website, license verification is triggered and new report is stored on your server.<br><br>
    Smart Reports ensure that only one report will be generated a day. In other words, if client performs hundreds of license verifications 7 days in a row, only 7 reports will be stored in database. This feature has no effect for properly activated licenses because verification is only performed every X days/weeks/months (defined by you) and doesn't cause excessive connections and reports.",

    "What is Envato API token and how to obtain it?"=>"Envato API token is used by <a href=\"https://codecanyon.net/item/envato-purchase-verification-plugin-for-auto-php-licenser/21943361?ref=phpmillion\" target=\"_blank\">Envato Purchase Verification Plugin</a> to fully automate licensing for authors who sell their products on Envato. This is how it works:<br><br>
    <ul>
    <li>User is asked to enter his purchase code before using application. If code doesn’t exist in $PRODUCT_NAME database, plugin looks for a matching purchase on Envato. If purchase exists, plugin issues a new license for corresponding product and application starts working. Otherwise, user can’t run protected application until he enters a valid purchase code.</li>
    <li>Plugin starts monitoring license on Envato. If purchase is cancelled or reversed, plugin automatically deactivates a corresponding license in $PRODUCT_NAME database and application stops working on user’s machine.</li>
    <li>In case of any event (license activation or deactivation, support contract renewal, connection failure, etc.) plugin generates comprehensive reports in $PRODUCT_NAME administration dashboard, so authors can track all activities easily.</li>
    </ul>
    API token can be obtained at <a href=\"https://build.envato.com/my-apps/\" target=\"_blank\">Manage Your Apps</a> section on Envato website. Click the <i>Create a new token</i> button and grant at least these permissions: <code>View and search Envato sites</code>, <code>View your items' sales history</code>, <code>Verify purchases of your items</code>. Once token is created, you will see its secret key. Copy/paste this token into the <i>Envato API Token</i> field at <i>Software Settings > Advanced Settings</i> tab and save the changes.",
    );



    //troubleshooting
    $troubleshooting=array(
    "License can't be issued because of invalid license type"=>"If error message 'Invalid license type (personal license can't use license code)' is displayed when issuing new license, it means personal and anonymous licenses are mixed together. In other words, if client's profile is selected from the list (personal license needs to be issued), license code should be empty because it's only used to verify anonymous licenses. Alternatively, if license code is entered (anonymous license needs to be issued), client's profile should not be selected.",

    "License can't be issued because of invalid domain"=>"If error message 'Invalid domain' is displayed when issuing new license, most likely licensed domain is entered wrong way. For example, it can't be IP-based (like http://1.1.1.1), include scheme (like http://www.clientdomain.com), or slash ( / ) symbol at the end (like http://clientdomain.com/path). In other words, it should be entered like clientdomain.com or clientdomain.com/path.",

    "Domain is already in use by another client"=>"In order to prevent any possible piracy, installation domain is locked to person who installed protected script on specific domain first. For example, if client A deploys a script protected by $PRODUCT_NAME on domain.com, client B will see error message when trying to install his copy of script on the same domain (because it's natural that client B should never have access to client's A domain).<br><br>
    However, in some very rare circumstances, there might be a situation when client B needs to replace existing installation on client's A domain with his own script. In case that happens, simply delete old installation using the <i>Licenses > View Licenses</i> tab, so client B can deploy his copy of script on \"locked\" domain.",

    "Maximum number of installations reached"=>"Once the maximum number of allowed installations for individual license is reached, new installations are blocked. If client needs to perform more more installations, the limit can be increased using the <i>Licenses > View Licenses</i> tab, or one of old installations should be deleted using the same tab. Deleting installation record in database is enough; even if user doesn't remove old files from his system, old installation will stop working within next automated license verification.",

    "Client's IP address was changed and license verification failed"=>"Each installation is locked to a specific IP address used to perform this installation. Once IP address gets changed, license verification fails (so even if someone clones all the protected files and databases, this illegal copy will not work). However, there might be a scenario when client's IP gets changed. In case that happens, simply update installation IP address using the <i>Installations > View Installations</i> tab, so blocked installation continues working.",

    "Record can't be added or updated"=>"If error message 'Invalid record details or duplicated record (no new data)' is displayed when adding new or updating existing record, it means this record is already stored in database. For example, it may happen when adding a new product that uses the same SKU as another product, adding a new client with email address already used by another client, adding a new license of the same product to the same client again, and so on. In other words, this error prevents you from adding duplicated data to you database; therefore, double-check record details and fix them accordingly.",

    "Some licenses don't display client's data"=>"Since $PRODUCT_NAME allows issuing anonymous licenses, it's perfectly normal that no client's data is displayed for this type of licenses (because there isn't any). However, anonymous licenses always contain unique license code in client's field.",

    "Some callbacks and/or reports don't display product name or client's data"=>"This only might happen when invalid product ID is hardcoded into script on user's machine, or script is trying to check a license which doesn't exist. In both cases, it's solely developer's mistake, which can be fixed by re-configuring script properly.",

    "License can't be installed or verified because of invalid signature"=>"The most common issue of invalid signature is inaccurate time on your server (where $PRODUCT_NAME is installed) and/or inaccurate time on user's machine (where your script is installed). It doesn't matter which timezone is used, but operating system time must always be accurate. For example, if actual time in your timezone is ".date("Y-m-d H:i").", but computer has wrong time set to ".date("Y-m-d H:i", strtotime("+1 hour")).", license signature might be identified as invalid; therefore, make sure system time is accurate.<br><br>The same error might also occur when license code is entered along with licensed email during installation. Since license can be either personal (email-based) or anonymous (code-based), license code should always be empty for personal licenses.",

    "No callbacks are received"=>"If no license verification callbacks from specific installation (license) are displayed in reports section, there might be a few different reasons. For example, it's possible user simply deleted script from his server. Another very common reason: script isn't used at the moment. Script performs initial verifications during installation and first use, but what happens later?<br><br>
    Let's say script is configured to perform license verification every 7 days, and gets installed on ".date("Y-m-d").". Naturally, another verification will be performed on ".date("Y-m-d", strtotime("+7 days")).". However, if user doesn't launch script after ".date("Y-m-d", strtotime("+6 days")).", no callback will be made. In other words, even if script is installed (bot not used) for months, no callbacks will be available to reduce system resources usage. However, license verification will be triggered the same second script gets used again (even if user does so years later).",

    "License is disabled, but script still works"=>"In order to reduce resources usage on your server, automated license verification is performed every X days/weeks/months (as defined by you inside script configuration). For example, if script is configured to verify license every 7 days, last license verification was performed on ".date("Y-m-d", strtotime("-1 day"))." and license was deactivated on ".date("Y-m-d").", script will stop working within next license check on ".date("Y-m-d", strtotime("+6 days")).".",

    "Not all files are deleted when invalid license is detected"=>"If $PRODUCT_NAME is configured to automatically delete all files and MySQL data when illegal license or possible cracking attempt is detected, but some files still remain, it means there's a problem with your files structure. According to documentation, directory <code>/SCRIPT</code> (can be renamed into anything you like) should always be placed in the root directory of your application. Put simply, if your application is installed at <code>/your/application/here</code>, $PRODUCT_NAME files should be located at <code>/your/application/here/SCRIPT</code> for auto-delete function to work properly.",
    );


    $collapse_no=1;
    foreach ($$array_to_use as $key=>$value)
        {
        $item_array['help_topic_title']=$key;
        $item_array['help_topic_text']=$value;
        $item_array['help_topic_class']="panel box box-$help_topic_class";
        $item_array['data_parent']="$array_to_use";
        $item_array['collapse_id']="collapse_$array_to_use"."$collapse_no";

        $collapse_no++;

        $root_array[]=$item_array;
        }

    return $root_array;
    }


$help_using_application_array=returnHelpTopicsArray($PRODUCT_NAME, $ROOT_URL, "using_application", "success");
$help_using_api_array=returnHelpTopicsArray($PRODUCT_NAME, $ROOT_URL, "using_api", "warning");
$help_protecting_your_script_array=returnHelpTopicsArray($PRODUCT_NAME, $ROOT_URL, "protecting_your_script", "danger");
$help_faq_array=returnHelpTopicsArray($PRODUCT_NAME, $ROOT_URL, "faq", "info");
$help_troubleshooting_array=returnHelpTopicsArray($PRODUCT_NAME, $ROOT_URL, "troubleshooting", "default");


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
