<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="Edit License";
$page_message="Edit license to be used. Update license information and click the 'Submit' button.<br><br>If IP address and/or domain is set, product will only work on specified IP and/or domain. If licensed domain is entered as clientdomain.com, product will work on clientdomain.com and clientdomain.com/any/directory. If licensed domain is entered as clientdomain.com/path, product will only work on clientdomain.com/path. It's possible to add multiple licensed IPs and/or domains by separating them with comma (,) symbol. If installations limit is set, client will not be able to run more copies of licensed product than specified number.<br><br>If expiration date is set, application will stop working after this date (expiration date can be updated at any time).";
$page_message_class="alert alert-info";
$page_header_file_no_data="licenses_view.php";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_GET) && is_array($_GET) && array_walk($_GET, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all GET values
    {
    extract($_GET, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (empty($license_id) || !filter_var($license_id, FILTER_VALIDATE_INT) || empty($rows_array=fetchRow("SELECT * FROM apl_licenses WHERE license_id=?", array($license_id), array("i")))) //invalid record
    {
    header("Location: $page_header_file_no_data");
    exit();
    }


if (!isset($submit_ok)) //extract fetched variables only if form wasn't submitted (otherwise data entered by user will be overwritten with data from database in case of form submission failure)
    {
    foreach ($rows_array as $row)
        {
        extract($row);
        }
    }


if (isset($submit_ok)) //code between {} tags is identical in files with the same name in /apl_admin and /apl_api directories, EXCEPT header("Location: $page_header_file_no_data"); LINE
    {
    if (!empty($delete_record) && $delete_record==1)
        {
        $removed_records=deleteRow("DELETE FROM apl_licenses WHERE license_id=?", array($license_id), array("i"));
        if ($removed_records>0)
            {
            $page_message="Deleted $removed_records license(s) from the database.";
            createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
            header("Location: $page_header_file_no_data");
            exit();
            }
        else
            {
            $error_detected=1;
            $error_details.="Invalid record or database error.<br>";
            }
        }

    if (filter_var($product_id, FILTER_VALIDATE_INT) && validateNumberOrRange($license_require_domain, 0, 1) && validateNumberOrRange($license_status, 0, 2))
        {
        if (empty($client_id) || !filter_var($client_id, FILTER_VALIDATE_INT)) //in case no client_id was submitted, its value must be stored as NULL in database
            {
            $client_id=null;
            }

        if (empty($license_code)) //in case no license_code was submitted, its value must be stored as NULL in database
            {
            $license_code=null;
            }

        if (!filter_var($client_id, FILTER_VALIDATE_INT) && empty($license_code))
            {
            $error_detected=1;
            $error_details.="Invalid client or license code.<br>";
            }

        if (filter_var($client_id, FILTER_VALIDATE_INT) && !empty($license_code))
            {
            $error_detected=1;
            $error_details.="Invalid license type (license can only be either personal (client-based) or anonymous (code-based)).<br>";
            }

        if (!empty($license_ip))
            {
            $license_ip_array=explode(",", $license_ip);
            foreach ($license_ip_array as $license_ip_array_key=>$license_ip_array_value)
                {
                if (!filter_var($license_ip_array_value, FILTER_VALIDATE_IP))
                    {
                    $error_detected=1;
                    $error_details.="Invalid IP address.<br>";
                    break;
                    }
                }
            }

        if (!empty($license_domain))
            {
            $license_domain_array=explode(",", $license_domain);
            foreach ($license_domain_array as $license_domain_array_key=>$license_domain_array_value)
                {
                if (!validateRawDomain(getRawDomain($license_domain_array_value)) || filter_var($license_domain_array_value, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED) || !ctype_alnum(substr($license_domain_array_value, -1))) //invalid TLD, scheme included, or last symbol is not alphanumeric (most likely ends with / or another non-alphanumeric character)
                    {
                    $error_detected=1;
                    $error_details.="Invalid domain.<br>";
                    break;
                    }
                }
            }

        if (!empty($license_limit) && !filter_var($license_limit, FILTER_VALIDATE_INT))
            {
            $error_detected=1;
            $error_details.="Invalid installations limit.<br>";
            }

        if (!empty($license_expire_date) && !verifyDate($license_expire_date, "Y-m-d"))
            {
            $error_detected=1;
            $error_details.="Invalid license expiration date.<br>";
            }

        if (!empty($license_updates_date) && !verifyDate($license_updates_date, "Y-m-d"))
            {
            $error_detected=1;
            $error_details.="Invalid updates expiration date.<br>";
            }

        if (!empty($license_support_date) && !verifyDate($license_support_date, "Y-m-d"))
            {
            $error_detected=1;
            $error_details.="Invalid support expiration date.<br>";
            }

        if ($error_detected!=1)
            {
            if (empty($license_envato) || !filter_var($license_envato, FILTER_VALIDATE_INT))
                {
                $license_envato=0;
                }

            if ($license_status==1)
                {
                $license_cancel_date="0000-00-00";
                }
            else
                {
                $license_cancel_date=$rows_array[0]['license_cancel_date']; //get old license_cancel_date, so this value can be used when license was deactivated previously and its status wasn't changed now
                if (empty($license_cancel_date) || !verifyDate($license_cancel_date, "Y-m-d")) //set cancel date to now only if no previous cancel date set
                    {
                    $license_cancel_date=date("Y-m-d");
                    }
                }

            $updated_records=updateRow("UPDATE apl_licenses SET license_order_number=?, license_ip=?, license_domain=?, license_require_domain=?, license_limit=?, license_cancel_date=?, license_expire_date=?, license_updates_date=?, license_support_date=?, license_comments=?, license_envato=?, license_status=? WHERE license_id=?", array($license_order_number, $license_ip, $license_domain, $license_require_domain, $license_limit, $license_cancel_date, $license_expire_date, $license_updates_date, $license_support_date, $license_comments, $license_envato, $license_status, $license_id), array("s", "s", "s", "i", "i", "s", "s", "s", "s", "s", "i", "i", "i"));
            if ($updated_records<1)
                {
                $error_detected=1;
                $error_details.="Invalid record details or duplicated record (no new data).<br>";
                }
            else
                {
                $action_success=1;

                $rows_array=fetchRow("SELECT * FROM apl_licenses LEFT JOIN apl_products ON apl_licenses.product_id=apl_products.product_id LEFT JOIN apl_clients ON apl_clients.client_id=apl_licenses.client_id WHERE apl_licenses.license_id=?", array($license_id), array("i")); //fetch product and client details to be used in reports
                foreach ($rows_array as $row)
                    {
                    extract($row);
                    }

                $client_formatted=formatClient($client_id, $client_fname, $client_lname, $client_email, $license_code);
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid product or license status.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="$product_title license for $client_formatted updated.";
        createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="The database could not be updated because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }
    }


//set default values for essential variables (mostly submitted to dropdown functions) when no values are set or values need to be reset
if (empty($license_limit) || !filter_var($license_limit, FILTER_VALIDATE_INT))
    {
    $license_limit=null;
    }

if (empty($license_expire_date) || !verifyDate($license_expire_date, "Y-m-d"))
    {
    $license_expire_date=null;
    }

if (empty($license_updates_date) || !verifyDate($license_updates_date, "Y-m-d"))
    {
    $license_updates_date=null;
    }

if (empty($license_support_date) || !verifyDate($license_support_date, "Y-m-d"))
    {
    $license_support_date=null;
    }


$products_array=returnProductsDropdownDisabledArray($product_id);
$clients_array=returnClientsDropdownDisabledArray($client_id);


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
