<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="Administrator Profile";
$page_message="Edit administrator profile. Update profile information and click the 'Submit' button.<br><br><b>Attention</b>: enter new password only if you want to change current password. Otherwise, leave password fields empty.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (isset($submit_ok))
    {
    if (filter_var($admin_email_1, FILTER_VALIDATE_EMAIL) && filter_var($admin_email_2, FILTER_VALIDATE_EMAIL) && validateNumberOrRange($admin_data_authenticity, 0, 1))
        {
        if ($admin_email_1!=$admin_email_2)
            {
            $error_detected=1;
            $error_details.="Email addresses do not match.<br>";
            }

        if (!empty($admin_password_1) || !empty($admin_password_2))
            {
            if ($admin_password_1!=$admin_password_2)
                {
                $error_detected=1;
                $error_details.="Passwords do not match.<br>";
                }

            if (strlen($admin_password_1)<5 || strlen($admin_password_2)<5)
                {
                $error_detected=1;
                $error_details.="Passwords are too short.<br>";
                }
            }

        if ($error_detected!=1)
            {
            if (empty($admin_password_1)) //no new password entered, use old one
                {
                $admin_password_1=$logged_admin_password;
                }
            else
                {
                $admin_password_1=password_hash($admin_password_1, PASSWORD_DEFAULT);
                }

            $logged_admin_data_authenticity=$admin_data_authenticity;

            $updated_records=updateRow("UPDATE apl_admins SET admin_email=?, admin_password=?, admin_data_authenticity=? WHERE admin_id=?", array($admin_email_1, $admin_password_1, $logged_admin_data_authenticity, $logged_admin_id), array("s", "s", "i", "i"));
            if ($updated_records<1)
                {
                $error_detected=1;
                $error_details.="Invalid record details or duplicated record (no new data).<br>";
                }
            else
                {
                $action_success=1;
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid email or protection status.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="Administrator profile updated.";
        createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="The database could not be updated because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }
    }


//set default values for essential variables (mostly submitted to dropdown functions) when no values are set or values need to be reset
if (empty($admin_email_1) || !filter_var($admin_email_1, FILTER_VALIDATE_EMAIL))
    {
    $admin_email_1=$logged_admin_email;
    }

if (empty($admin_email_2) || !filter_var($admin_email_2, FILTER_VALIDATE_EMAIL))
    {
    $admin_email_2=$logged_admin_email;
    }

if (empty($admin_data_authenticity) || !validateNumberOrRange($admin_data_authenticity, 0, 1))
    {
    $admin_data_authenticity=$logged_admin_data_authenticity;
    }


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
