<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="Email Settings";
$page_message="Configure reminder email settings, enable and disable individual options.<br><br><b>Attention</b>: reminder emails will only be sent to personal (email-based) license owners who have their email addresses set.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (isset($submit_ok))
    {
    if (!empty($email_from_name) && filter_var($email_from_address, FILTER_VALIDATE_EMAIL) && validateNumberOrRange($email_expiring_license_days, 0, 30) && validateNumberOrRange($email_expiring_updates_days, 0, 30) && validateNumberOrRange($email_expiring_support_days, 0, 30))
        {
        if ($error_detected!=1)
            {
            $updated_records=updateRow("UPDATE apl_settings SET EMAIL_FROM_NAME=?, EMAIL_FROM_ADDRESS=?, EMAIL_EXPIRING_LICENSE_DAYS=?, EMAIL_EXPIRING_UPDATES_DAYS=?, EMAIL_EXPIRING_SUPPORT_DAYS=?", array($email_from_name, $email_from_address, $email_expiring_license_days, $email_expiring_updates_days, $email_expiring_support_days), array("s", "s", "i", "i", "i"));
            if ($updated_records<1)
                {
                $error_detected=1;
                $error_details.="Invalid settings or duplicated record (no new data).<br>";
                }
            else
                {
                $action_success=1;
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid email fields or time range.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="Email settings updated.";
        createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="The database could not be updated because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }


    //get all settings again
    $rows_array=fetchRow("SELECT * FROM apl_settings");
    foreach ($rows_array as $row)
        {
        extract($row);
        }
    }


//set default values for essential variables (mostly submitted to dropdown functions) when no values are set or values need to be reset
if (!empty($email_from_name))
    {
    $EMAIL_FROM_NAME=$email_from_name;
    }

if (!empty($email_from_address))
    {
    $EMAIL_FROM_ADDRESS=$email_from_address;
    }


$email_expiring_license_days_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30), "Days", "Disabled", $EMAIL_EXPIRING_LICENSE_DAYS);
$email_expiring_updates_days_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30), "Days", "Disabled", $EMAIL_EXPIRING_UPDATES_DAYS);
$email_expiring_support_days_array=returnNumbersDropdownArray(array(0, 1, 7, 14, 30), "Days", "Disabled", $EMAIL_EXPIRING_SUPPORT_DAYS);


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
