<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");
require_once(__DIR__."/login_check.php");


$page_title="General Settings";
$page_message="Configure general software settings, enable and disable individual options.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


if (isset($submit_ok))
    {
    if (!empty($timezone) && validateNumberOrRange($records_on_admin_page, 10, 500) && validateNumberOrRange($records_on_index_page, 1, 10) && validateNumberOrRange($records_on_search_page, 10, 500) && validateNumberOrRange($smart_reports, 0, 1) && validateNumberOrRange($smart_tables, 0, 1))
        {
        if ($error_detected!=1)
            {
            $updated_records=updateRow("UPDATE apl_settings SET TIMEZONE=?, RECORDS_ON_ADMIN_PAGE=?, RECORDS_ON_INDEX_PAGE=?, RECORDS_ON_SEARCH_PAGE=?, SMART_REPORTS=?, SMART_TABLES=?", array($timezone, $records_on_admin_page, $records_on_index_page, $records_on_search_page, $smart_reports, $smart_tables), array("s", "i", "i", "i", "i", "i"));
            if ($updated_records<1)
                {
                $error_detected=1;
                $error_details.="Invalid settings or duplicated record (no new data).<br>";
                }
            else
                {
                $action_success=1;
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid settings.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="General software settings updated.";
        createReport(strip_tags($page_message), $logged_admin_id, 1, $error_detected);
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="The database could not be updated because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }


    //get all settings again
    $rows_array=fetchRow("SELECT * FROM apl_settings");
    foreach ($rows_array as $row)
        {
        extract($row);
        }
    }


$timezones_array=returnTimezonesArray($TIMEZONE);
$records_on_admin_page_array=returnNumbersDropdownArray(array(10, 25, 50, 100, 200, 500), "Records", "Disabled", $RECORDS_ON_ADMIN_PAGE);
$records_on_index_page_array=returnNumbersDropdownArray(array(1, 3, 5, 10), "Records", "Disabled", $RECORDS_ON_INDEX_PAGE);
$records_on_search_page_array=returnNumbersDropdownArray(array(10, 25, 50, 100, 200, 500), "Records", "Disabled", $RECORDS_ON_SEARCH_PAGE);


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
