<?php
//used by script on user's machine to check if user's updates are active
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_settings.php");


//get IP, refer, requested page, script filename, and user agent (browser)
if (isset($_SERVER['REMOTE_ADDR'])) {$ip_address=$_SERVER['REMOTE_ADDR'];}
if (isset($_SERVER['HTTP_REFERER'])) {$refer=$_SERVER['HTTP_REFERER'];}
if (isset($_SERVER['REQUEST_URI'])) {$requested_page=$_SERVER['REQUEST_URI'];}
if (isset($_SERVER['SCRIPT_FILENAME'])) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);}
if (isset($_SERVER['HTTP_USER_AGENT'])) {$user_agent=$_SERVER['HTTP_USER_AGENT'];}


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


//check basic data
if (filter_var($ip_address, FILTER_VALIDATE_IP) && in_array($user_agent, $SUPPORTED_BROWSERS_ARRAY) && filter_var($product_id, FILTER_VALIDATE_INT) && filter_var($root_url, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED) && $root_url==$refer && !empty($installation_hash) && $installation_hash==hash("sha256", $root_url.$client_email.$license_code) && !empty($license_signature))
    {
    $notification_case=null;

    if (filter_var($client_email, FILTER_VALIDATE_EMAIL)) //search for email-based license first
        {
        $license_rows=fetchRow("SELECT * FROM apl_licenses JOIN apl_products ON apl_licenses.product_id=apl_products.product_id JOIN apl_clients ON apl_licenses.client_id=apl_clients.client_id WHERE apl_products.product_id=? AND apl_clients.client_email=? AND apl_licenses.client_id IS NOT NULL AND apl_licenses.license_code IS NULL", array($product_id, $client_email), array("i", "s"));
        }

    if (empty($license_rows) && !empty($license_code)) //email-based license does not exist (license results array is empty), search for code-based license (only if code was submitted)
        {
        $license_rows=fetchRow("SELECT * FROM apl_licenses JOIN apl_products ON apl_licenses.product_id=apl_products.product_id WHERE apl_products.product_id=? AND apl_licenses.client_id IS NULL AND apl_licenses.license_code IS NOT NULL AND apl_licenses.license_code=?", array($product_id, $license_code), array("i", "s"));
        }

    if (empty($license_rows)) //neither email-based or code-based license exists, set variables (that were not submitted from user's module, but are used in notifications) to "null" to prevent "undefined variable" errors
        {
        $client_fname=null;
        $client_lname=null;
        $product_title=null;
        $product_description=null;
        $product_url_homepage=null;
        $product_url_download=null;
        $product_version=null;
        $license_expire_date=null;
        $license_cancel_date=null;
        $license_updates_date=null;
        $license_support_date=null;
        $license_limit=null;

        $error_detected=1;
        $error_details=setValue($error_details, "license or licensed product not found or is inactive");
        $notification_case=setValue($notification_case, "notification_license_not_found");

        echo returnLicenseNotification($notification_case, $root_url, $ip_address, $client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit);
        exit();
        }


    foreach ($license_rows as $row) //fetch license details
        {
        extract($row);
        }


    if (!filter_var($client_id, FILTER_VALIDATE_INT)) //client_id and client_email were set to null and empty for anonymous license, so other client_ variables were not fetched from apl_clients table. set variables used in notifications to "null"
        {
        $client_fname=null;
        $client_lname=null;
        }
    $installation_domain=getRootUrl("$root_url/", 1, 1, 0, 1); //make url without scheme, www. and / at the end because this type of url is stored on server (add / at the end before processing because software stores root url without /)
    $client_formatted=formatClient($client_id, $client_fname, $client_lname, $client_email, $license_code);


    if (!verifyScriptSignature($license_signature, $product_id, $root_url, $client_email, $license_code)) //invalid signature
        {
        $error_detected=1;
        $error_details=setValue($error_details, "invalid license signature");
        $notification_case=setValue($notification_case, "notification_invalid_signature");
        }

    if ($license_status==0) //license cancelled
        {
        $error_detected=1;
        $error_details=setValue($error_details, "license cancelled on $license_cancel_date");
        $notification_case=setValue($notification_case, "notification_license_cancelled");
        }

    if ($license_status==2) //license suspended
        {
        $error_detected=1;
        $error_details=setValue($error_details, "license suspended");
        $notification_case=setValue($notification_case, "notification_license_suspended");
        }

    if (verifyDate($license_expire_date, "Y-m-d") && $license_expire_date<=date("Y-m-d")) //license expired
        {
        $error_detected=1;
        $error_details=setValue($error_details, "license expired on $license_expire_date");
        $notification_case=setValue($notification_case, "notification_license_expired");
        }

    if (!empty($license_ip))
        {
        $license_ip_array=explode(",", $license_ip);
        if (!in_array($ip_address, $license_ip_array)) //invalid IP
            {
            $error_detected=1;
            $error_details=setValue($error_details, "IP address $ip_address is not allowed");
            $notification_case=setValue($notification_case, "notification_invalid_ip");
            }
        }

    if (!empty($license_domain))
        {
        $license_domain_detected=0; //will be changed to 1 later only if everything OK
        $license_domain_array=explode(",", $license_domain);
        foreach ($license_domain_array as $license_domain_array_key=>$license_domain_array_value)
            {
            if (stristr(getRootUrl("$root_url/", 1, 1, 0, 1), $license_domain_array_value)) //check if URL (where script is installed) matches one of allowed URLs (add / at the end before processing because software stores root url without /)
                {
                $license_domain_detected=1;
                break;
                }
            }

        if ($license_domain_detected!=1) //invalid domain
            {
            $error_detected=1;
            $error_details=setValue($error_details, "domain $root_url is invalid");
            $notification_case=setValue($notification_case, "notification_invalid_domain");
            }
        }

    if ($license_require_domain==1 && filter_var(getRawDomain($root_url), FILTER_VALIDATE_IP) || $license_require_domain==1 && !filter_var($root_url, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED)) //domain required
        {
        $error_detected=1;
        $error_details=setValue($error_details, "a real domain is required");
        $notification_case=setValue($notification_case, "notification_domain_required");
        }

    if (!empty($installation_domain)) //check if installation on this domain and IP is not performed by someone else
        {
        if (filter_var($client_id, FILTER_VALIDATE_INT)) //client_id is not null (so license_code is null), search by client_id (prepared statements has limitations when using where clauses with null values)
            {
            $third_party_installation_rows=fetchRow("SELECT * FROM apl_installations WHERE product_id=? AND installation_ip=? AND installation_domain=? AND (client_id IS NULL OR client_id!=?)", array($product_id, $ip_address, $installation_domain, $client_id), array("i", "s", "s", "i"));
            }
        else //client_id is null (so license_code is not null), search by license_code (prepared statements has limitations when using where clauses with null values)
            {
            $third_party_installation_rows=fetchRow("SELECT * FROM apl_installations WHERE product_id=? AND installation_ip=? AND installation_domain=? AND (license_code IS NULL OR license_code!=?)", array($product_id, $ip_address, $installation_domain, $license_code), array("i", "s", "s", "s"));
            }

        if (!empty($third_party_installation_rows)) //this domain is used by another client
            {
            $error_detected=1;
            $error_details=setValue($error_details, "product $product_title is already installed on domain $root_url ($ip_address) by another client");
            $notification_case=setValue($notification_case, "notification_domain_in_use");
            }
        }

    if ($license_limit!=0) //check if installations limit is not exceeded
        {
        if (filter_var($client_id, FILTER_VALIDATE_INT)) //client_id is not null (so license_code is null), search by client_id (prepared statements has limitations when using where clauses with null values)
            {
            $client_concurrent_installations_row=fetchRow("SELECT * FROM apl_installations WHERE product_id=? AND client_id=? AND (installation_ip!=? OR installation_domain!=?)", array($product_id, $client_id, $ip_address, $installation_domain), array("i", "i", "s", "s"));
            }
        else //client_id is null (so license_code is not null), search by license_code (prepared statements has limitations when using where clauses with null values)
            {
            $client_concurrent_installations_row=fetchRow("SELECT * FROM apl_installations WHERE product_id=? AND license_code=? AND (installation_ip!=? OR installation_domain!=?)", array($product_id, $license_code, $ip_address, $installation_domain), array("i", "s", "s", "s"));
            }

        if (count($client_concurrent_installations_row)>=$license_limit) //client already performed maximum allowed number of installations on OTHER domains, so this one would exceed limit
            {
            $error_detected=1;
            $error_details=setValue($error_details, "maximum installations limit ($license_limit) reached");
            $notification_case=setValue($notification_case, "notification_license_limit");
            }
        }

    if (verifyDate($license_updates_date, "Y-m-d") && $license_updates_date<=date("Y-m-d")) //updates expired - THIS LINE SHOULD ONLY BE USED IN LINCENSE_UPDATES.PHP AND SOFTWARE_DOWNLOAD.PHP FILES
        {
        $error_detected=1;
        $error_details=setValue($error_details, "updates expired on $license_updates_date");
        $notification_case=setValue($notification_case, "notification_updates_expired");
        }

    /*if (verifyDate($license_support_date, "Y-m-d") && $license_support_date<=date("Y-m-d")) //support expired - THIS LINE SHOULD ONLY BE USED IN LINCENSE_SUPPORT.PHP FILE
        {
        $error_detected=1;
        $error_details=setValue($error_details, "support expired on $license_support_date");
        $notification_case=setValue($notification_case, "notification_support_expired");
        }*/


    if ($error_detected!=1) //everything OK so far, do final check if license is really active
        {
        if ($license_status==1) //license active, check if installation is stored in database and active too
            {
            if (filter_var($client_id, FILTER_VALIDATE_INT)) //client_id is not null (so license_code is null), search by client_id (prepared statements has limitations when using where clauses with null values)
                {
                $this_installation_row=fetchRow("SELECT * FROM apl_installations WHERE product_id=? AND client_id=? AND installation_ip=? AND installation_domain=? AND installation_hash=? AND installation_status=?", array($product_id, $client_id, $ip_address, $installation_domain, $installation_hash, 1), array("i", "i", "s", "s", "s", "i"));
                }
            else //client_id is null (so license_code is not null), search by license_code (prepared statements has limitations when using where clauses with null values)
                {
                $this_installation_row=fetchRow("SELECT * FROM apl_installations WHERE product_id=? AND license_code=? AND installation_ip=? AND installation_domain=? AND installation_hash=? AND installation_status=?", array($product_id, $license_code, $ip_address, $installation_domain, $installation_hash, 1), array("i", "s", "s", "s", "s", "i"));
                }

            if (!empty($this_installation_row)) //installation exists and is active
                {
                $action_success=1;
                $report_text="$product_title installation for $client_formatted on $installation_domain ($ip_address) is eligible for updates.";
                }
            else //installation does not exist or is inactive
                {
                $error_detected=1;
                $error_details=setValue($error_details, "installation does not exist or is inactive");
                $notification_case=setValue($notification_case, "notification_installation_not_found");
                }
            }
        }


    if ($action_success==1) //everything OK (license_signature will also be displayed in this case)
        {
        $notification_case=setValue($notification_case, "notification_license_ok"); //$report_text is already created above in this case
        }
    else
        {
        $report_text="$product_title installation for $client_formatted on $installation_domain ($ip_address) is not eligible for updates because of this reason: $error_details."; //$notification_case is already created above in this case
        }


    echo returnLicenseNotification($notification_case, $root_url, $ip_address, $client_email, $client_fname, $client_lname, $license_code, $product_id, $product_title, $product_description, $product_url_homepage, $product_url_download, $product_version, $license_expire_date, $license_cancel_date, $license_updates_date, $license_support_date, $license_limit);
    createLicenseReport($SMART_REPORTS, $product_id, $client_id, $license_code, $report_text, $error_detected);
    }
