<?php
require_once(__DIR__."/../apl_config.php");
require_once(__DIR__."/../apl_ver.php");
require_once(__DIR__."/../apl_modules/phpmillion_core.php");
require_once(__DIR__."/../apl_modules/phpmillion_modules.php");
require_once(__DIR__."/../apl_modules/phpmillion_plugins.php");
require_once(__DIR__."/../apl_modules/phpmillion_plugins_gui.php");
require_once(__DIR__."/../apl_modules/apl_core_configuration.php");
require_once(__DIR__."/../apl_modules/apl_core_functions.php");
require_once(__DIR__."/../lib/swiftmailer/swift_required.php");
require_once(__DIR__."/../lib/Twig/Autoloader.php");
require_once(__DIR__."/../lib/html-compress-twig/autoload.php");

//array with form fields that allow HTML-like tags (where key is script filename and value is a sub-array with names of form fields that allow HTML)
$FORM_FIELDS_WITH_TAGS=array();

//get IP, refer, requested page, script filename, and user agent (browser)
if (isset($_SERVER['REMOTE_ADDR'])) {$ip_address=$_SERVER['REMOTE_ADDR'];}
if (isset($_SERVER['HTTP_REFERER'])) {$refer=$_SERVER['HTTP_REFERER'];}
if (isset($_SERVER['REQUEST_URI'])) {$requested_page=$_SERVER['REQUEST_URI'];}
if (isset($_SERVER['SCRIPT_FILENAME'])) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);}
if (isset($_SERVER['HTTP_USER_AGENT'])) {$user_agent=$_SERVER['HTTP_USER_AGENT'];}


$page_title="Installation";
$page_message="Enter your email address (will be used for administrator's profile), <a href='https://help.market.envato.com/hc/en-us/articles/202822600-Where-Is-My-Purchase-Code-' target='_blank'>purchase code</a>, installation URL, and click the 'Submit' button.";
$page_message_class="alert alert-info";


$action_success=0; //will be changed to 1 later only if everything OK
$error_detected=0; //will be changed to 1 later if error occurs
$error_details=null; //will be filled with errors (if any)
$added_records=0;
$updated_records=0;
$removed_records=0;


if (!empty($_POST) && is_array($_POST) && array_walk($_POST, "sanitizeSubmittedData", array("script_filename"=>$script_filename, "html_fields"=>$FORM_FIELDS_WITH_TAGS))) //sanitize super variable with all POST values
    {
    extract($_POST, EXTR_SKIP); //extract sanitized data (don't overwrite existing variables)
    }


//do database connectivity, server compatibility and latest version checks
if (!isset($submit_ok)) //do checks only before form is submitted, no need to display it after running installer
    {
    $fatal_error_detected=0;
    $fatal_error_details=null;
    $compatibility_issue_detected=0;
    $compatibility_check_missing_component=null;
    $ROOT_URL=str_replace("/".basename(__DIR__), "", getRootUrl(getCurrentUrl(), 0, 0, 0, 1)); //get current URL

    if (!$GLOBALS["mysqli"])
        {
        $fatal_error_detected=1;
        $fatal_error_details.="Invalid MySQL connection details<br>";

        $page_message="<b>Attention!</b> Installation wizard could not connect to your MySQL database. Check your database connection details in apl_config.php file and try again.";
        $page_message_class="alert alert-danger";
        }

    if ($fatal_error_detected!=1) //do compatibility checks only if database connection can be established
        {
        if (version_compare(PHP_VERSION, "5.5.0", "<"))
            {
            $compatibility_issue_detected=1;
            $compatibility_check_missing_component.="PHP 5.5 or higher<br>";
            }
        if (!function_exists("mysqli_stmt_get_result"))
            {
            $compatibility_issue_detected=1;
            $compatibility_check_missing_component.="MySQLi (with mysqlnd driver)<br>";
            }
        if (!function_exists("curl_init"))
            {
            $compatibility_issue_detected=1;
            $compatibility_check_missing_component.="cURL<br>";
            }
        if (!function_exists("openssl_encrypt"))
            {
            $compatibility_issue_detected=1;
            $compatibility_check_missing_component.="OpenSSL<br>";
            }

        if ($compatibility_issue_detected==1)
            {
            $page_message="<b>Attention!</b> These PHP modules are not enabled on your server:<br><br>$compatibility_check_missing_component<br>Enable modules listed above and run the installer again.<br><br><b>Attention!</b> This error comes right from your server (<b>not</b> $PRODUCT_NAME); therefore, <b>only server administrator can fix it</b>.";
            $page_message_class="alert alert-danger";
            }
        }

    if ($fatal_error_detected!=1 && $compatibility_issue_detected!=1)
        {
        $version_check_content=customPost(APL_ROOT_URL."/apl_callbacks/version_check.php", $ROOT_URL, "product_id=".rawurlencode(APL_PRODUCT_ID)."&product_version=".rawurlencode($PRODUCT_VERSION)."&connection_hash=".rawurlencode(hash("sha256", "version_check")));
        if (!empty($version_check_content))
            {
            $latest_product_version=aplParseXmlTags($version_check_content, "version_check");
            if (!empty($latest_product_version)) //latest version parsed successfully
                {
                if ($latest_product_version!=$PRODUCT_VERSION) //new version available
                    {
                    $page_message="<b>Attention!</b> You are trying to install $PRODUCT_NAME version $PRODUCT_VERSION, while the latest version is $latest_product_version. <a href='https://codecanyon.net/downloads?ref=phpmillion' target='_blank'>Download the latest version</a> and try again.";
                    $page_message_class="alert alert-danger";
                    }
                }
            else //parsing failed
                {
                $page_message="<b>Attention!</b> Impossible to parse the latest $PRODUCT_NAME version. If you can open <a href='$PRODUCT_HOMEPAGE' target='_blank'>$PRODUCT_NAME website</a> via browser, but this error is still displayed, <a href='https://www.phpmillion.com/forums/' target='_blank'>contact our support team</a>. Otherwise, try again later.";
                $page_message_class="alert alert-danger";
                }
            }
        else //no connection
            {
            $page_message="<b>Attention!</b> Impossible to connect to $PRODUCT_NAME server. If you can open <a href='$PRODUCT_HOMEPAGE' target='_blank'>$PRODUCT_NAME website</a> via browser, but this error is still displayed, <a href='https://www.phpmillion.com/forums/' target=_blank'>contact our support team</a>. Otherwise, try again later.";
            $page_message_class="alert alert-danger";
            }
        }
    }


if (isset($submit_ok))
    {
    if (filter_var($CLIENT_EMAIL, FILTER_VALIDATE_EMAIL) && filter_var($ROOT_URL, FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED) && ctype_alnum(substr($ROOT_URL, -1)))
        {
        if ($error_detected!=1)
            {
            aplVerifyEnvatoPurchase($LICENSE_CODE);

            $license_notifications_array=aplInstallLicense($ROOT_URL, $CLIENT_EMAIL, $LICENSE_CODE, $GLOBALS["mysqli"]); //install license
            if ($license_notifications_array['notification_case']=="notification_license_ok")
                {
                $INSTALLATION_HASH=hash("sha256", $ROOT_URL.$CLIENT_EMAIL.$LICENSE_CODE); //generate hash

                $post_info_mysql="product_id=".rawurlencode(APL_PRODUCT_ID)."&product_version=".rawurlencode($PRODUCT_VERSION)."&client_email=".rawurlencode($CLIENT_EMAIL)."&license_code=".rawurlencode($LICENSE_CODE)."&root_url=".rawurlencode($ROOT_URL)."&installation_hash=".rawurlencode($INSTALLATION_HASH)."&license_signature=".rawurlencode(aplGenerateScriptSignature($ROOT_URL, $CLIENT_EMAIL, $LICENSE_CODE)); //format variables used to get MySQL query
                $product_install_query=customPost(APL_ROOT_URL."/apl_callbacks/mysql_install.php", $ROOT_URL, $post_info_mysql);
                if (!empty($product_install_query))
                    {
                    $notifications_array=aplParseServerNotifications($product_install_query); //parse <notification_case> along with message
                    if (!empty($notifications_array['notification_case']) && !empty($notifications_array['notification_text'])) //a specific error notification returned
                        {
                        $error_detected=1;
                        $error_details.=$notifications_array['notification_text'].".<br>";
                        }
                    else //raw installation query returned
                        {
                        $mysql_bad_array=array("_ROOT_URL_", "_CLIENT_EMAIL_", "_LICENSE_CODE_", "_INSTALLATION_HASH_", "_ADMIN_PASSWORD_", "_CURRENT_DATE_", "_DATABASE_VERSION_");
                        $mysql_good_array=array($ROOT_URL, $CLIENT_EMAIL, $LICENSE_CODE, $INSTALLATION_HASH, password_hash("phpmillion", PASSWORD_DEFAULT), date("Y-m-d"), $PRODUCT_VERSION);
                        $product_install_query=str_replace($mysql_bad_array, $mysql_good_array, $product_install_query); //replace some variables with actual values

                        mysqli_close($GLOBALS["mysqli"]); //close MySQL connection and connect again to prevent errors when mysqli_query is executed again (1st query was executed by aplInstallLicense() function)
                        sleep(1);

                        $GLOBALS["mysqli"]=mysqli_connect($DB_HOST, $DB_USER, $DB_PASS, $DB_NAME, $DB_PORT);
                        mysqli_query($GLOBALS["mysqli"], "SET SESSION sql_mode='NO_ENGINE_SUBSTITUTION'");
                        mysqli_set_charset($GLOBALS["mysqli"], "utf8");

                        mysqli_multi_query($GLOBALS["mysqli"], $product_install_query) or die(mysqli_error($GLOBALS["mysqli"])); //run main installation query
                        $action_success=1; //if script wasn't aborted by die() function above, it means everything was just fine
                        }
                    }
                else
                    {
                    $error_detected=1;
                    $error_details.="Impossible to connect to $PRODUCT_NAME server to parse installation query.<br>";
                    }
                }
            else //something went wrong
                {
                $error_detected=1;
                $error_details.=$license_notifications_array['notification_text'];
                }
            }
        }
    else
        {
        $error_detected=1;
        $error_details.="Invalid email address or installation URL.<br>";
        }

    if ($action_success==1) //everything OK
        {
        $page_message="$PRODUCT_NAME successfully installed.<br><br>Now delete /apl_install directory and <a href='$ROOT_URL/apl_admin'><b>click here</b></a> to enter administration dashboard. Use email address <b>$CLIENT_EMAIL</b> and password <b>phpmillion</b> to login.";
        $page_message_class="alert alert-success";
        }
    else //display error message
        {
        $page_message="$PRODUCT_NAME could not be installed because of this error: <br><br>$error_details";
        $page_message_class="alert alert-danger";
        }
    }


//Twig templating starts
if (!isset($script_filename)) {$script_filename=basename($_SERVER['SCRIPT_FILENAME']);} //if $script_filename is not set yet (usually set in login_check.php), get it now (will be used in Twig forms)

Twig_Autoloader::register();
$loader=new Twig_Loader_Filesystem("../apl_templates"); //load files from templates directory
$twig=new Twig_Environment($loader); //create Twig environment

$twig->addExtension(new \nochso\HtmlCompressTwig\Extension());

echo $twig->render(basename(__DIR__)."/".basename(__FILE__, ".php").".twig", get_defined_vars()); //render requested template
//Twig templating ends
