<?php
//slightly modified modules shared by most phpmillion apps


//create report
function createReport($report_text, $account_id, $report_system, $report_failure)
    {
    $report_date=date("Y-m-d");

    insertRow("INSERT IGNORE INTO apl_reports (account_id, report_date, report_text, report_system, report_failure) VALUES (?, ?, ?, ?, ?)", array($account_id, $report_date, $report_text, $report_system, $report_failure), array("i", "s", "s", "i", "i"));
    }


//remove old data
function cleanupDatabase($DATABASE_CLEANUP_ENABLED, $DATABASE_CLEANUP_CALLBACKS, $DATABASE_CLEANUP_REPORTS_MAIN, $DATABASE_CLEANUP_REPORTS_SYSTEM, $DATABASE_CLEANUP_LICENSES, $DATABASE_CLEANUP_DATE)
    {
    $removed_records_total=0;

    if ($DATABASE_CLEANUP_ENABLED==1 && $DATABASE_CLEANUP_DATE<date("Y-m-d")) //need to launch it only once a day
        {
        $report_text="Automatic database cleanup module launched. These records were removed: ";

        if (filter_var($DATABASE_CLEANUP_CALLBACKS, FILTER_VALIDATE_INT))
            {
            $old_date_allowed=date("Y-m-d", strtotime("-$DATABASE_CLEANUP_CALLBACKS days")); //callbacks older than this will be removed

            $removed_records=deleteRow("DELETE FROM apl_callbacks WHERE callback_date<?", array($old_date_allowed), array("s"));
            $removed_records_total+=$removed_records;

            $report_text.="callbacks older than $DATABASE_CLEANUP_CALLBACKS day(s) ($removed_records records), ";
            }

        if (filter_var($DATABASE_CLEANUP_REPORTS_MAIN, FILTER_VALIDATE_INT))
            {
            $old_date_allowed=date("Y-m-d", strtotime("-$DATABASE_CLEANUP_REPORTS_MAIN days")); //main reports older than this will be removed

            $removed_records=deleteRow("DELETE FROM apl_reports WHERE report_system=? AND report_date<?", array(0, $old_date_allowed), array("i", "s"));
            $removed_records_total+=$removed_records;

            $report_text.="main reports older than $DATABASE_CLEANUP_REPORTS_MAIN day(s) ($removed_records records), ";
            }

        if (filter_var($DATABASE_CLEANUP_REPORTS_SYSTEM, FILTER_VALIDATE_INT))
            {
            $old_date_allowed=date("Y-m-d", strtotime("-$DATABASE_CLEANUP_REPORTS_SYSTEM days")); //system reports older than this will be removed

            $removed_records=deleteRow("DELETE FROM apl_reports WHERE report_system=? AND report_date<?", array(1, $old_date_allowed), array("i", "s"));
            $removed_records_total+=$removed_records;

            $report_text.="system reports older than $DATABASE_CLEANUP_REPORTS_SYSTEM day(s) ($removed_records records), ";
            }

        if (filter_var($DATABASE_CLEANUP_LICENSES, FILTER_VALIDATE_INT))
            {
            $old_date_allowed=date("Y-m-d", strtotime("-$DATABASE_CLEANUP_LICENSES days")); //inactive licenses older than this will be removed

            $removed_records=deleteRow("DELETE FROM apl_licenses WHERE license_cancel_date!=? AND license_cancel_date<?", array("0000-00-00", $old_date_allowed), array("s", "s"));
            $removed_records_total+=$removed_records;

            $report_text.="inactive licenses older than $DATABASE_CLEANUP_LICENSES day(s) ($removed_records records), ";
            }

        if ($removed_records_total<1) //no records removed, generate a different report
            {
            $report_text="Automatic database cleanup module launched. No unused records were found  "; //2 last symbols will be cut when generating report
            }

        $database_cleanup_date=date("Y-m-d");
        updateRow("UPDATE apl_settings SET DATABASE_CLEANUP_DATE=?", array($database_cleanup_date), array("s"));
        createReport(substr($report_text, 0, -2).".", 0, 1, 0);
        }
    }


//send notification email to admin(s)
function emailAdmin($email_subject, $email_text)
    {
    global $PRODUCT_NAME;

    $mailer_result=null;

    if (!empty($email_subject) && !empty($email_text))
        {
        $transport=Swift_MailTransport::newInstance(); //create transport
        $mailer=Swift_Mailer::newInstance($transport); //create mailer using created transport
        $message=Swift_Message::newInstance("$PRODUCT_NAME: $email_subject") //create message
        ->setBody("This notification was sent by $PRODUCT_NAME to inform you about this event:<br><br>".nl2br($email_text), "text/html");

        $rows_array=fetchRow("SELECT * FROM apl_admins");
        foreach ($rows_array as $row)
            {
            extract($row);

            if (filter_var($admin_email, FILTER_VALIDATE_EMAIL))
                {
                $message->setFrom(array($admin_email=>$PRODUCT_NAME));
                $message->setTo(array($admin_email=>"$PRODUCT_NAME Admin"));

                $mailer_result=$mailer->send($message); //send message
                }
            }
        }

    return $mailer_result;
    }


//parse and display RSS feeds
function parseDisplayRss($NEWS_TEXT, $feed_url, $last_parsing_date, $days_between_parsing, $items_to_parse)
    {
    if (empty($NEWS_TEXT) || !verifyDate($last_parsing_date, "Y-m-d") || getDaysBetweenDates($last_parsing_date, date("Y-m-d"))>=$days_between_parsing) //time to parse RSS feed
        {
        $parsed_results_array=array();
        $rss_feeds_array=array();
        $content=customGet($feed_url, "", "");
        if (!empty($content))
            {
            $content_object=@simplexml_load_string($content);
            if (is_object($content_object))
                {
                foreach ($content_object as $entry)
                    {
                    foreach ($entry->item as $item)
                        {
                        $record['title']=(string) cleanupContent($item->title);
                        $record['full_url']=(string) trim(strip_tags($item->link));
                        $record['date']=(string) date("Y-m-d", strtotime(trim(strip_tags($item->pubDate))));
                        $parsed_results_array[]=$record;
                        }
                    }
                unset($content_object); //prevent leaking memory
                }

            if (!empty($parsed_results_array)) //results parsed, make sub-arrays of titles, descriptions and links
                {
                foreach ($parsed_results_array as $key=>$value)
                    {
                    if (empty($value['title']) || !filter_var($value['full_url'], FILTER_VALIDATE_URL, FILTER_FLAG_SCHEME_REQUIRED | FILTER_FLAG_HOST_REQUIRED))
                        {
                        unset($parsed_results_array[$key]);
                        }
                    }

                if (!empty($parsed_results_array)) //entries still exist after cleanup
                    {
                    $NEWS_TEXT=json_encode(array_slice($parsed_results_array, 0, $items_to_parse));
                    }
                }
            }
        $NEWS_DATE=date("Y-m-d"); //update data even if no entries were parsed, so endless parsing doesn't occur

        updateRow("UPDATE apl_settings SET NEWS_TEXT=?, NEWS_DATE=?", array($NEWS_TEXT, $NEWS_DATE), array("s", "s"));
        }

    if (!empty($NEWS_TEXT)) //display stored entries
        {
        $NEWS_TEXT=json_decode($NEWS_TEXT, true); //decode feed entries and convert into arrays
        if (!empty($NEWS_TEXT))
            {
            foreach ($NEWS_TEXT as $item)
                {
                $item['title']=truncateText($item['title'], 40, "symbols", 0, 2);
                $item['full_url']=$item['full_url'];
                $item['date']=$item['date'];
                $rss_feeds_array[]=$item;
                }
            }
        }
    else
        {
        $item['title']="No announcements";
        $rss_feeds_array[]=$item;
        }

    return $rss_feeds_array;
    }


//display sidebar menu
function returnLeftMenuArray($script_filename)
    {
    $left_menu_array[]=array("title"=>"Products", "target"=>"#", "icon"=>"fa-shopping-cart", "sub_menu"=>array(array("title"=>"Add New Product", "target"=>"products_add.php"), array("title"=>"View Products", "target"=>"products_view.php", "target_sub"=>"products_edit.php")));
    $left_menu_array[]=array("title"=>"Clients", "target"=>"#", "icon"=>"fa-users", "sub_menu"=>array(array("title"=>"Add New Client", "target"=>"clients_add.php"), array("title"=>"View Clients", "target"=>"clients_view.php", "target_sub"=>"clients_edit.php")));
    $left_menu_array[]=array("title"=>"Licenses", "target"=>"#", "icon"=>"fa-id-card-o", "sub_menu"=>array(array("title"=>"Add New License", "target"=>"licenses_add.php"), array("title"=>"View Licenses", "target"=>"licenses_view.php", "target_sub"=>"licenses_edit.php")));
    $left_menu_array[]=array("title"=>"Installations", "target"=>"#", "icon"=>"fa-hdd-o", "sub_menu"=>array(array("title"=>"View Installations", "target"=>"installations_view.php", "target_sub"=>"installations_edit.php")));
    $left_menu_array[]=array("title"=>"Callbacks", "target"=>"#", "icon"=>"fa-phone-square", "sub_menu"=>array(array("title"=>"View Callbacks", "target"=>"callbacks_view.php")));
    $left_menu_array[]=array("title"=>"Reports", "target"=>"#", "icon"=>"fa-line-chart", "sub_menu"=>array(array("title"=>"View License Reports", "target"=>"reports_license_view.php"), array("title"=>"View System Reports", "target"=>"reports_system_view.php")));
    $left_menu_array[]=array("title"=>"Server Notifications", "target"=>"#", "icon"=>"fa-commenting-o", "sub_menu"=>array(array("title"=>"Customize Notifications", "target"=>"notifications_edit.php"), array("title"=>"Customize Emails", "target"=>"emails_edit.php")));
    $left_menu_array[]=array("title"=>"Software Settings", "target"=>"#", "icon"=>"fa-cogs", "sub_menu"=>array(array("title"=>"General Settings", "target"=>"settings_general.php"), array("title"=>"Advanced Settings", "target"=>"settings_advanced.php"), array("title"=>"Email Settings", "target"=>"settings_email.php"), array("title"=>"Database Cleanup Settings", "target"=>"settings_cleanup.php")));
    $left_menu_array[]=array("title"=>"API Keys", "target"=>"#", "icon"=>"fa-key", "sub_menu"=>array(array("title"=>"Add New API Key", "target"=>"api_keys_add.php"), array("title"=>"View API Keys", "target"=>"api_keys_view.php", "target_sub"=>"api_keys_edit.php")));
    $left_menu_array[]=array("title"=>"Banned Hosts", "target"=>"#", "icon"=>"fa-ban", "sub_menu"=>array(array("title"=>"Add New Banned Host", "target"=>"banned_hosts_add.php"), array("title"=>"View Banned Hosts", "target"=>"banned_hosts_view.php", "target_sub"=>"banned_hosts_edit.php")));
    $left_menu_array[]=array("title"=>"Extra Tools", "target"=>"#", "icon"=>"fa-wrench", "sub_menu"=>array(array("title"=>"Configuration Generator", "target"=>"config_generate.php")));

    foreach ($left_menu_array as $left_menu_item)
        {
        $item['title']=$left_menu_item['title'];
        $item['class']="treeview";
        $item['target']=$left_menu_item['target'];
        $item['icon']=$left_menu_item['icon'];
        $item['sub_menu']=array();

        foreach ($left_menu_item['sub_menu'] as $left_menu_sub_item) //format submenus for each entry
            {
            $sub_item['title']=$left_menu_sub_item['title'];
            $sub_item['class']=null;
            $sub_item['target']=$left_menu_sub_item['target'];

            if (!empty($left_menu_sub_item['target']) && $script_filename==$left_menu_sub_item['target'] || !empty($left_menu_sub_item['target_sub']) && $script_filename==$left_menu_sub_item['target_sub']) //mark this menu as active in sidebar
                {
                $item['class']="treeview active";
                $sub_item['class']="active";
                }

            $item['sub_menu'][]=$sub_item;
            }

        $sidebar_items_array[]=$item;
        }

    return $sidebar_items_array;
    }


//prevent banned hosts from using system
function blockBannedHosts($BANNED_HOSTS)
    {
    if ($BANNED_HOSTS==1 && isset($_SERVER['REMOTE_ADDR']))
        {
        $ip_address=$_SERVER['REMOTE_ADDR'];

        $rows_array=fetchRow("SELECT * FROM apl_banned_hosts WHERE banned_host_ip=?", array($ip_address), array("s"));
        if (!empty($rows_array))
            {
            foreach ($rows_array as $row)
                {
                extract($row);
                }

            $banned_host_blocks++;
            $banned_host_block_date=date("Y-m-d");

            updateRow("UPDATE apl_banned_hosts SET banned_host_blocks=?, banned_host_block_date=? WHERE banned_host_id=?", array($banned_host_blocks, $banned_host_block_date, $banned_host_id), array("i", "s", "i"));

            $notification_case="notification_host_banned";

            echo returnLicenseNotification($notification_case, null, $ip_address, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
            createReport("Host $ip_address is banned and was blocked.", 0, 1, 2);
            exit();
            }
        }
    }
